# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.10.1
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# + [markdown] papermill={"duration": 0.006585, "end_time": "2023-06-07T20:53:05.294467", "exception": false, "start_time": "2023-06-07T20:53:05.287882", "status": "completed"} tags=[]
# <link rel="preconnect" href="https://fonts.gstatic.com">
# <link href="https://fonts.googleapis.com/css2?family=Fira+Code&display=swap" rel="stylesheet">
#
# ### License
#
# <p style="font-family: 'Fira Code', monospace; font-size: 1.2rem">
# Copyright (c) 2021-2022 Xilinx, Inc.<br>
# Copyright (c) 2022-2023 Advanced Micro Devices, Inc.<br><br>
# Licensed under the Apache License, Version 2.0 (the "License");<br>
# you may not use this file except in compliance with the License.<br><br>
# You may obtain a copy of the License at <a href="http://www.apache.org/licenses/LICENSE-2.0"?>http://www.apache.org/licenses/LICENSE-2.0</a><br><br>
# Unless required by applicable law or agreed to in writing, software<br>
# distributed under the License is distributed on an "AS IS" BASIS,<br>
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.<br>
# See the License for the specific language governing permissions and<br>
# limitations under the License.<br>
# </p>
#

# + [markdown] papermill={"duration": 0.005077, "end_time": "2023-06-07T20:53:05.305373", "exception": false, "start_time": "2023-06-07T20:53:05.300296", "status": "completed"} tags=[]
# ILA Advanced Trigger Example
# =========================

# + [markdown] papermill={"duration": 0.004813, "end_time": "2023-06-07T20:53:05.315104", "exception": false, "start_time": "2023-06-07T20:53:05.310291", "status": "completed"} tags=[]
# Description
# -----------
# This demo shows how to use the ILA Advanced Trigger Mode.
# In this mode, the trigger setup is described in text using the ILA Trigger State Machine (TSM) language.
# See UG908 Vivado Design User Guide: Programming and Debugging, Appendix B "Trigger State Machine Language Description". 
#
#
# Requirements
# ------------
# The following is required to run this demo:
# 1. Local or remote access to a Versal device
# 2. 2022.2 cs_server and hw_server applications
# 3. Python 3.8 environment
# 4. A clone of the chipscopy git enterprise repository:
#    - https://gitenterprise.xilinx.com/chipscope/chipscopy
#
# ---

# + [markdown] papermill={"duration": 0.004828, "end_time": "2023-06-07T20:53:05.324835", "exception": false, "start_time": "2023-06-07T20:53:05.320007", "status": "completed"} tags=[]
# ## Step 1 - Set up environment

# + papermill={"duration": 0.354278, "end_time": "2023-06-07T20:53:05.683989", "exception": false, "start_time": "2023-06-07T20:53:05.329711", "status": "completed"} tags=[]
import os
from enum import Enum
import chipscopy
from chipscopy import create_session, get_design_files, null_callback, report_versions
from chipscopy.api.ila import ILAStatus, ILAWaveform
from io import StringIO
from pprint import pformat

# + papermill={"duration": 0.011312, "end_time": "2023-06-07T20:53:05.701336", "exception": false, "start_time": "2023-06-07T20:53:05.690024", "status": "completed"} tags=[]
# Specify locations of the running hw_server and cs_server below.
CS_URL = os.getenv("CS_SERVER_URL", "TCP:localhost:3042")
HW_URL = os.getenv("HW_SERVER_URL", "TCP:localhost:3121")

# + papermill={"duration": 0.01079, "end_time": "2023-06-07T20:53:05.717517", "exception": false, "start_time": "2023-06-07T20:53:05.706727", "status": "completed"} tags=[]
design_files = get_design_files("vck190/production/chipscopy_ced")
PROGRAMMING_FILE = design_files.programming_file
PROBES_FILE = design_files.probes_file
assert os.path.isfile(PROGRAMMING_FILE)
assert os.path.isfile(PROBES_FILE)

# + papermill={"duration": 0.010282, "end_time": "2023-06-07T20:53:05.732932", "exception": false, "start_time": "2023-06-07T20:53:05.722650", "status": "completed"} tags=[]
print(f"HW_URL={HW_URL}")
print(f"CS_URL={CS_URL}")
print(f"PDI={PROGRAMMING_FILE}")
print(f"LTX={PROBES_FILE}")

# + [markdown] papermill={"duration": 0.004874, "end_time": "2023-06-07T20:53:05.743248", "exception": false, "start_time": "2023-06-07T20:53:05.738374", "status": "completed"} tags=[]
# ## Step 2 - Create a session and connect to the server(s)
# Here we create a new session and print out some versioning information for diagnostic purposes.
# The session is a container that keeps track of devices and debug cores.

# + papermill={"duration": 0.93485, "end_time": "2023-06-07T20:53:06.683102", "exception": false, "start_time": "2023-06-07T20:53:05.748252", "status": "completed"} tags=[]
print(f"Using chipscopy api version: {chipscopy.__version__}")
session = create_session(cs_server_url=CS_URL, hw_server_url=HW_URL)
report_versions(session)

# + [markdown] papermill={"duration": 0.0056, "end_time": "2023-06-07T20:53:06.695399", "exception": false, "start_time": "2023-06-07T20:53:06.689799", "status": "completed"} tags=[]
# ## Step 3 - Get our device from the session

# + papermill={"duration": 0.356562, "end_time": "2023-06-07T20:53:07.057725", "exception": false, "start_time": "2023-06-07T20:53:06.701163", "status": "completed"} tags=[]
# Use the first available device and setup its debug cores
if len(session.devices) == 0:
    raise ValueError("No devices detected")
print(f"Device count: {len(session.devices)}")
versal_device = session.devices.get(family="versal")

# + [markdown] papermill={"duration": 0.005457, "end_time": "2023-06-07T20:53:07.069992", "exception": false, "start_time": "2023-06-07T20:53:07.064535", "status": "completed"} tags=[]
# ## Step 4 - Program the device with our example programming file

# + papermill={"duration": 5.285838, "end_time": "2023-06-07T20:53:12.361516", "exception": false, "start_time": "2023-06-07T20:53:07.075678", "status": "completed"} tags=[]
print(f"Programming {PROGRAMMING_FILE}...")
versal_device.program(PROGRAMMING_FILE)

# + [markdown] papermill={"duration": 0.00557, "end_time": "2023-06-07T20:53:12.374128", "exception": false, "start_time": "2023-06-07T20:53:12.368558", "status": "completed"} tags=[]
# ## Step 5 - Detect Debug Cores

# + papermill={"duration": 0.490627, "end_time": "2023-06-07T20:53:12.870530", "exception": false, "start_time": "2023-06-07T20:53:12.379903", "status": "completed"} tags=[]
print(f"Discovering debug cores...")
versal_device.discover_and_setup_cores(ltx_file=PROBES_FILE)

# + papermill={"duration": 2.516872, "end_time": "2023-06-07T20:53:15.393942", "exception": false, "start_time": "2023-06-07T20:53:12.877070", "status": "completed"} tags=[]
ila_count = len(versal_device.ila_cores)
print(f"\nFound {ila_count} ILA cores in design")

# + papermill={"duration": 0.012122, "end_time": "2023-06-07T20:53:15.412438", "exception": false, "start_time": "2023-06-07T20:53:15.400316", "status": "completed"} tags=[]
if ila_count == 0:
    print("No ILA core found! Exiting...")
    raise ValueError("No ILA cores detected")

# + papermill={"duration": 0.020919, "end_time": "2023-06-07T20:53:15.438960", "exception": false, "start_time": "2023-06-07T20:53:15.418041", "status": "completed"} tags=[]
# List all detected ILA Cores
ila_cores = versal_device.ila_cores
for index, ila_core in enumerate(ila_cores):
    print(f"    ILA Core #{index}: NAME={ila_core.name}, UUID={ila_core.core_info.uuid}")

# + papermill={"duration": 0.01378, "end_time": "2023-06-07T20:53:15.458682", "exception": false, "start_time": "2023-06-07T20:53:15.444902", "status": "completed"} tags=[]
# Get the ILA cores matching a given name. filter_by returns a list, even if just one item is present.
my_ila = versal_device.ila_cores.filter_by(name="chipscopy_i/counters/ila_slow_counter_0")[0]

print(f"USING ILA: {my_ila.name}")

# + [markdown] papermill={"duration": 0.005787, "end_time": "2023-06-07T20:53:15.470254", "exception": false, "start_time": "2023-06-07T20:53:15.464467", "status": "completed"} tags=[]
# ## Step 6 - Get Information for this ILA Core
# Note:
# - 'has_advanced_trigger' is True. This ILA supports the advanced trigger feature.
# - 'tsm_counter_widths' shows 4 counters of bit width 16.

# + papermill={"duration": 0.013505, "end_time": "2023-06-07T20:53:15.489534", "exception": false, "start_time": "2023-06-07T20:53:15.476029", "status": "completed"} tags=[]
print("\nILA Name:", my_ila.name)
print("\nILA Core Info", my_ila.core_info)
print("\nILA Static Info", my_ila.static_info)


# + [markdown] papermill={"duration": 0.005833, "end_time": "2023-06-07T20:53:15.500973", "exception": false, "start_time": "2023-06-07T20:53:15.495140", "status": "completed"} tags=[]
# ## Step 7 -  Trigger Immediately using Advanced Trigger Mode
#
# Trigger State Machine trigger descriptions may be in a text file, or in a io.StringIO object.
# Some of the ILA status information applies only to Advanced Trigger Mode:
# - tsm_counters
# - tsm_flags
# - tsm_state
# - tsm_state_name

# + papermill={"duration": 1.075478, "end_time": "2023-06-07T20:53:16.582772", "exception": false, "start_time": "2023-06-07T20:53:15.507294", "status": "completed"} tags=[]
TRIGGER_NOW_TSM = StringIO(
"""
    state my_state0:
        trigger;
"""
)

# Note, if the TSM text is in a file, you can give the file path string as the first argument.
my_ila.run_advanced_trigger(TRIGGER_NOW_TSM, trigger_position=0, window_count=1, window_size=8)


my_ila.refresh_status()
print("\nILA Status:\n")
print("Trigger State Machine counter values:      ", my_ila.status.tsm_counters)
print("Trigger State Machine flags:               ", my_ila.status.tsm_flags)
print("Trigger State Machine current state index: ", my_ila.status.tsm_state)
print("Trigger State Machine current state name : ", my_ila.status.tsm_state_name)

# + [markdown] papermill={"duration": 0.005681, "end_time": "2023-06-07T20:53:16.594850", "exception": false, "start_time": "2023-06-07T20:53:16.589169", "status": "completed"} tags=[]
# ## Step 8 - Upload Captured Waveform
# Wait at most half a minutes, for ILA to trigger and capture data.

# + papermill={"duration": 0.248508, "end_time": "2023-06-07T20:53:16.849682", "exception": false, "start_time": "2023-06-07T20:53:16.601174", "status": "completed"} tags=[]
my_ila.wait_till_done(max_wait_minutes=0.5)
my_ila.upload()
if not my_ila.waveform:
    print("\nUpload failed!")

# + [markdown] papermill={"duration": 0.00574, "end_time": "2023-06-07T20:53:16.861879", "exception": false, "start_time": "2023-06-07T20:53:16.856139", "status": "completed"} tags=[]
# ## Step 9 - Print samples for probe 'chipscopy_i/counters/slow_counter_0_Q_1'. 
#
# Using the function ILAWaveform.get_data(), the waveform data is put into a sorted dict.
# First 4 entries in sorting order are: trigger, sample index, window index, window sample index.
# Then comes probe values. In this case just one probe.

# + papermill={"duration": 0.013936, "end_time": "2023-06-07T20:53:16.881732", "exception": false, "start_time": "2023-06-07T20:53:16.867796", "status": "completed"} tags=[]
counter_probe_name = 'chipscopy_i/counters/slow_counter_0_Q_1'

def print_probe_values(waveform: ILAWaveform, probe_names: [str]):
    samples = waveform.get_data(
        probe_names,
        include_trigger=True,
        include_sample_info=True,
    )
    for trigger, sample_index, window_index, window_sample_index, value in zip(*samples.values()):
        trigger = "<-- Trigger" if trigger else ""
        print(
            f"Window:{window_index}  Window Sample:{window_sample_index}  dec:{value:10}  hex:0x{value:08X} {trigger}"
        )

print_probe_values(my_ila.waveform, [counter_probe_name])


# + [markdown] papermill={"duration": 0.005743, "end_time": "2023-06-07T20:53:16.893221", "exception": false, "start_time": "2023-06-07T20:53:16.887478", "status": "completed"} tags=[]
# ## Step 10 - Check if TSM is Valid
# - The TSM below has undefined probe names and undefined states.
# - Use "compile_only=True" argument when just checking if the TSM text is valid.
# - The run_advanced_trigger() function returns a tuple with 2 values: "error_count" and "error_message".

# + papermill={"duration": 0.037641, "end_time": "2023-06-07T20:53:16.937211", "exception": false, "start_time": "2023-06-07T20:53:16.899570", "status": "completed"} tags=[]
TSM_WITH_ERRORS = StringIO(
    """
    state state_a:
      if (chipscopy_ex_i/counters/slow_counter_0_Q_1 == 32'hXX33_0000 &&
          chipscopy_i/counters/slow_sine_Dout >= 'habcd) then
        trigger;
      elseif (chipscopy_i/counters/slow_sine_Dout >= 'habcd) then
        reset_counter $counter0;
        goto state_2;
      else
        reset_counter $counter1;
        goto state_3;
      endif
    state state_b:
      if (chipscopy_i/counters23/slow_counter_0_Q_1 == 32'h3333_0000) then
        trigger;
      else
        reset_counter $counter3;
        goto state_2;
      endif
    state state_c:
      goto state_3;
    """
)

# Check the TSM text for errors, using "compile_only=True
#
error_count, error_message = my_ila.run_advanced_trigger(TSM_WITH_ERRORS, compile_only=True)
print(f'\n\nThe Advanced Trigger State machine "TSM_WITH_ERRORS" has {error_count} error(s).'
      f'\n\n{error_message}')


# + [markdown] papermill={"duration": 0.006143, "end_time": "2023-06-07T20:53:16.949533", "exception": false, "start_time": "2023-06-07T20:53:16.943390", "status": "completed"} tags=[]
# ## Step 11 - Define a Status Progress Monitor Function
# Monitor TSM specific status, when ILA capture is active.

# + papermill={"duration": 0.010721, "end_time": "2023-06-07T20:53:16.966184", "exception": false, "start_time": "2023-06-07T20:53:16.955463", "status": "completed"} tags=[]
def status_progress(future):
    """Called in Event Thread"""
    st = future.progress

    if st.is_full:
        print(f"\nAll data has been captured.")
    else:
        print(f"State: {st.tsm_state_name}   Counters: {st.tsm_counters}    Flags: {st.tsm_flags}")       


# + [markdown] papermill={"duration": 0.006009, "end_time": "2023-06-07T20:53:16.978382", "exception": false, "start_time": "2023-06-07T20:53:16.972373", "status": "completed"} tags=[]
# ## Step 12 - Run Trigger State Machine with Flags and Counters
# In STATE_A:
# - Remain in STATE_A until hex value ending with "33_0000", has occurred 8 times (counter values 0-7).
# - Set \$flag0
# - Go to state STATE_B
#
# In STATE_B:
# - Count hex value ending with "AAA_BBBB" 10 times (counter values 0-9).
# - Then set \$flag1 and trigger.

# + papermill={"duration": 26.849485, "end_time": "2023-06-07T20:53:43.834419", "exception": false, "start_time": "2023-06-07T20:53:16.984934", "status": "completed"} tags=[]
TSM_FLAGS_COUNTERS = StringIO(
    f"""
    state STATE_A:
        if ( {counter_probe_name} == 32'hxx33_0000 && $counter0 == 'u7) then
            set_flag $flag0;
            goto STATE_B;
        elseif ( {counter_probe_name} == 32'hxx33_0000) then
            increment_counter $counter0;
            goto STATE_A;
        else
            goto STATE_A;
      endif
      
    state STATE_B:
        if ( {counter_probe_name} == 32'hxAAA_BBBB && $counter1 == 'u9) then
            set_flag $flag1;
            trigger;
        elseif ( {counter_probe_name} == 32'hxAAA_BBBB) then
            increment_counter $counter1;
            goto STATE_B;
        else
            goto STATE_B;
      endif
      
"""
)

my_ila.run_advanced_trigger(TSM_FLAGS_COUNTERS, window_size=8)

future = my_ila.monitor_status(max_wait_minutes=1.0, progress=status_progress, done=chipscopy.null_callback)

# future.result is blocking, until monitor_status() function has completed or timed-out or been cancelled.
# Meanwhile, the status_progress() function is called twice per second to print out status.
status = future.result
print(f"\nCounters: {status.tsm_counters}    Flags: {status.tsm_flags}")

# + [markdown] papermill={"duration": 0.006844, "end_time": "2023-06-07T20:53:43.848726", "exception": false, "start_time": "2023-06-07T20:53:43.841882", "status": "completed"} tags=[]
# ## Step 13 - Upload Captured Waveform

# + papermill={"duration": 0.170472, "end_time": "2023-06-07T20:53:44.025787", "exception": false, "start_time": "2023-06-07T20:53:43.855315", "status": "completed"} tags=[]
my_ila.upload()
if not my_ila.waveform:
    print("\nUpload failed!")

# + [markdown] papermill={"duration": 0.006881, "end_time": "2023-06-07T20:53:44.040592", "exception": false, "start_time": "2023-06-07T20:53:44.033711", "status": "completed"} tags=[]
# ## Step 14 - Print samples for probe 'chipscopy_i/counters/slow_counter_0_Q_1'. 

# + papermill={"duration": 0.013263, "end_time": "2023-06-07T20:53:44.060267", "exception": false, "start_time": "2023-06-07T20:53:44.047004", "status": "completed"} tags=[]
print_probe_values(my_ila.waveform, [counter_probe_name])

# + papermill={"duration": 0.006509, "end_time": "2023-06-07T20:53:44.073266", "exception": false, "start_time": "2023-06-07T20:53:44.066757", "status": "completed"} tags=[]




# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.10.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# + [markdown] papermill={"duration": 0.006301, "end_time": "2023-06-07T20:45:59.413034", "exception": false, "start_time": "2023-06-07T20:45:59.406733", "status": "completed"} tags=[]
# <link rel="preconnect" href="https://fonts.gstatic.com">
# <link href="https://fonts.googleapis.com/css2?family=Fira+Code&display=swap" rel="stylesheet">
#
# ### License
#
# <p style="font-family: 'Fira Code', monospace; font-size: 1.2rem">
# Copyright (c) 2021-2022 Xilinx, Inc.<br>
# Copyright (c) 2022-2023 Advanced Micro Devices, Inc.<br><br>
# Licensed under the Apache License, Version 2.0 (the "License");<br>
# you may not use this file except in compliance with the License.<br><br>
# You may obtain a copy of the License at <a href="http://www.apache.org/licenses/LICENSE-2.0"?>http://www.apache.org/licenses/LICENSE-2.0</a><br><br>
# Unless required by applicable law or agreed to in writing, software<br>
# distributed under the License is distributed on an "AS IS" BASIS,<br>
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.<br>
# See the License for the specific language governing permissions and<br>
# limitations under the License.<br>
# </p>
#

# + [markdown] papermill={"duration": 0.004897, "end_time": "2023-06-07T20:45:59.423517", "exception": false, "start_time": "2023-06-07T20:45:59.418620", "status": "completed"} tags=[]
# # IBERT link and eye scan example
#
#
# <img src="../../img/api_overview.png" width="500" align="left">

# + [markdown] papermill={"duration": 0.004707, "end_time": "2023-06-07T20:45:59.433232", "exception": false, "start_time": "2023-06-07T20:45:59.428525", "status": "completed"} tags=[]
# ## Description
# This example shows how to interact with the IBERT (Integrated Bit Error Ratio Tester) debug core service via ChipScoPy APIs.
# - Program the ChipScoPy CED design onto the XCVC1902 device on a VCK190 board
# - Verify that the expected IBERT quads are instantiated by the design
# - Create links and change link settings to get link lock
# - Run and plot eye scans for the links
# - Run a sweep on a link
#
#
# ## Requirements
# - Local or remote Xilinx Versal board, such as a VCK190
# - Xilinx hw_server 2022.2 installed and running
# - Xilinx cs_server 2022.2 installed and running
# - Python 3.8 or greater installed
# - ChipScoPy 2022.2 installed
# - Jupyter notebook support installed - Please do so, using the command `pip install chipscopy[jupyter]`
# - Plotting support installed - Please do so, using the command `pip install chipscopy[core-addons]`
# - Optional - [External loopback](https://www.samtec.com/kits/optics-fpga/hspce-fmcp/) (For the sweep example only).

# + [markdown] papermill={"duration": 0.00499, "end_time": "2023-06-07T20:45:59.443220", "exception": false, "start_time": "2023-06-07T20:45:59.438230", "status": "completed"} tags=[]
# ## 1 - Initialization: Imports and File Paths
#
# After this step,
# - Required functions and classes are imported
# - Paths to server(s) and files are set correctly

# + jupyter={"outputs_hidden": false} papermill={"duration": 0.339263, "end_time": "2023-06-07T20:45:59.787611", "exception": false, "start_time": "2023-06-07T20:45:59.448348", "status": "completed"} tags=[]
import os
from more_itertools import one
from itertools import product

from chipscopy import create_session, report_versions, report_hierarchy, get_design_files
from chipscopy.api.ibert.aliases import (
    EYE_SCAN_HORZ_RANGE,
    EYE_SCAN_VERT_RANGE,
    EYE_SCAN_VERT_STEP,
    EYE_SCAN_HORZ_STEP,
    EYE_SCAN_TARGET_BER,
    PATTERN,
    RX_LOOPBACK,
    TX_PRE_CURSOR,
    TX_POST_CURSOR,
    TX_DIFFERENTIAL_SWING,
    RX_TERMINATION_VOLTAGE,
    RX_COMMON_MODE
)
from chipscopy.api.ibert import create_eye_scans, create_links


# + papermill={"duration": 0.012661, "end_time": "2023-06-07T20:45:59.806227", "exception": false, "start_time": "2023-06-07T20:45:59.793566", "status": "completed"} tags=[]
# Make sure to start the hw_server and cs_server prior to running.
# Specify locations of the running hw_server and cs_server below.
# The default is localhost - but can be other locations on the network.
CS_URL = os.getenv("CS_SERVER_URL", "TCP:localhost:3042")
HW_URL = os.getenv("HW_SERVER_URL", "TCP:localhost:3121")

# The get_design_files() function tries to find the PDI and LTX files. In non-standard
# configurations, you can put the path for PROGRAMMING_FILE and PROBES_FILE below.
design_files = get_design_files("vck190/production/chipscopy_ced")

PDI_FILE = design_files.programming_file

print(f"PROGRAMMING_FILE: {PDI_FILE}")

# + [markdown] papermill={"duration": 0.004867, "end_time": "2023-06-07T20:45:59.816031", "exception": false, "start_time": "2023-06-07T20:45:59.811164", "status": "completed"} tags=[]
# ## 2 - Create a session and connect to the hw_server and cs_server
#
# The session is a container that keeps track of devices and debug cores.
# After this step,
# - Session is initialized and connected to server(s)
# - Versions are detected and reported to stdout

# + jupyter={"outputs_hidden": false} papermill={"duration": 0.996812, "end_time": "2023-06-07T20:46:00.817830", "exception": false, "start_time": "2023-06-07T20:45:59.821018", "status": "completed"} tags=[]
session = create_session(cs_server_url=CS_URL, hw_server_url=HW_URL)
report_versions(session)

# + [markdown] papermill={"duration": 0.005212, "end_time": "2023-06-07T20:46:00.829441", "exception": false, "start_time": "2023-06-07T20:46:00.824229", "status": "completed"} tags=[]
# ## 3 - Program the device with the example design
#
# After this step,
# - Device is programmed with the example programming file

# + jupyter={"outputs_hidden": false} papermill={"duration": 6.530379, "end_time": "2023-06-07T20:46:07.365047", "exception": false, "start_time": "2023-06-07T20:46:00.834668", "status": "completed"} tags=[]
# Typical case - one device on the board - get it.
device = session.devices.filter_by(family="versal").get()
device.program(PDI_FILE)

# + [markdown] papermill={"duration": 0.007113, "end_time": "2023-06-07T20:46:07.379026", "exception": false, "start_time": "2023-06-07T20:46:07.371913", "status": "completed"} tags=[]
# ## 4 - Discover and setup the IBERT core
#
# Debug core discovery initializes the chipscope server debug cores.
#
# After this step,
#
# - The cs_server is initialized and ready for use
# - The first ibert found is used

# + jupyter={"outputs_hidden": false} papermill={"duration": 2.17366, "end_time": "2023-06-07T20:46:09.558342", "exception": false, "start_time": "2023-06-07T20:46:07.384682", "status": "completed"} tags=[]
device.discover_and_setup_cores(ibert_scan=True)
print("--> Debug core discovery done")

if len(device.ibert_cores) == 0:
    print("No IBERT core found! Exiting...")
    exit()

# Use the first available IBERT core from the device
ibert = device.ibert_cores.at(index=0)

if len(ibert.gt_groups) == 0:
    print("No GT Groups available for use! Exiting...")
    exit()

print(f"GT Groups available - {[gt_group_obj.name for gt_group_obj in ibert.gt_groups]}")

# + [markdown] papermill={"duration": 0.005344, "end_time": "2023-06-07T20:46:09.570202", "exception": false, "start_time": "2023-06-07T20:46:09.564858", "status": "completed"} tags=[]
# ## 5 - Print the hierarchy of the IBERT core
#
# We also ensure that all the quads instantiated by the ChipScoPy CED design are found by the APIs

# + jupyter={"outputs_hidden": false} papermill={"duration": 21.55884, "end_time": "2023-06-07T20:46:31.135397", "exception": false, "start_time": "2023-06-07T20:46:09.576557", "status": "completed"} tags=[]
report_hierarchy(ibert)

assert len(ibert.gt_groups) == 4

q205 = one(ibert.gt_groups.filter_by(name="Quad_205"))
q204 = one(ibert.gt_groups.filter_by(name="Quad_204"))
q201 = one(ibert.gt_groups.filter_by(name="Quad_201"))
q105 = one(ibert.gt_groups.filter_by(name="Quad_105"))

# + [markdown] papermill={"duration": 0.006204, "end_time": "2023-06-07T20:46:31.148215", "exception": false, "start_time": "2023-06-07T20:46:31.142011", "status": "completed"} tags=[]
# ## 6 - Create links between following TXs and RXs
# - Quad 205 CH0 TX to Quad 205 CH0 RX
# - Quad 204 CH1 TX to Quad 204 CH1 RX
# - Quad 201 CH2 TX to Quad 201 CH2 RX
# - Quad 105 CH3 TX to Quad 105 CH3 RX

# + papermill={"duration": 0.01468, "end_time": "2023-06-07T20:46:31.169105", "exception": false, "start_time": "2023-06-07T20:46:31.154425", "status": "completed"} tags=[]
links = create_links(
    txs=[q205.gts[0].tx, q204.gts[1].tx, q201.gts[2].tx, q105.gts[3].tx],
    rxs=[q205.gts[0].rx, q204.gts[1].rx, q201.gts[2].rx, q105.gts[3].rx],
)

print("--> Done creating links")

# + [markdown] papermill={"duration": 0.00546, "end_time": "2023-06-07T20:46:31.180443", "exception": false, "start_time": "2023-06-07T20:46:31.174983", "status": "completed"} raw_mimetype="text/markdown" tags=[]
# ## 7 - Print the valid values for pattern and loopback, set the pattern for the TXs and RXs to "PRBS 31" and set loopback to "Near-End PMA"
#
# In order to lock the internal pattern checker, TX and RX patterns need to match. We also need to have some kind of loopback, internal/external.
#
# We are assuming that no external cable loopback is present and hence making use of internal loopback.

# + papermill={"duration": 7.9692, "end_time": "2023-06-07T20:46:39.155168", "exception": false, "start_time": "2023-06-07T20:46:31.185968", "status": "completed"} raw_mimetype="text/x-python" tags=[]
for link in links:
    print(f"\n----- {link.name} -----")
    _, tx_pattern_report = link.tx.property.report(link.tx.property_for_alias[PATTERN]).popitem()
    _, rx_pattern_report = link.rx.property.report(link.rx.property_for_alias[PATTERN]).popitem()
    _, rx_loopback_report = link.tx.property.report(
        link.rx.property_for_alias[RX_LOOPBACK]
    ).popitem()

    print(f"--> Valid values for TX pattern - {tx_pattern_report['Valid values']}")
    print(f"--> Valid values for RX pattern - {rx_pattern_report['Valid values']}")
    print(f"--> Valid values for RX loopback - {rx_loopback_report['Valid values']}")

    props = {link.tx.property_for_alias[PATTERN]: "PRBS 31"}
    link.tx.property.set(**props)
    link.tx.property.commit(list(props.keys()))

    props = {
        link.rx.property_for_alias[PATTERN]: "PRBS 31",
        link.rx.property_for_alias[RX_LOOPBACK]: "Near-End PMA",
    }
    link.rx.property.set(**props)
    link.rx.property.commit(list(props.keys()))
    print(f"\n--> Set both patterns to 'PRBS 31' & loopback to 'Near-End PMA' for {link}")

    assert link.rx.pll.locked and link.tx.pll.locked
    print(f"--> RX and TX PLLs are locked for {link}. Checking for link lock...")
    assert link.status != "No link"
    print(f"--> {link} is linked as expected")

# + [markdown] papermill={"duration": 0.006308, "end_time": "2023-06-07T20:46:39.169121", "exception": false, "start_time": "2023-06-07T20:46:39.162813", "status": "completed"} tags=[]
# ## 8 - Create eye scan objects for all the links, set the scan params and start the scan
#
# The eye scans will be run in parallel

# + papermill={"duration": 7.645677, "end_time": "2023-06-07T20:46:46.821265", "exception": false, "start_time": "2023-06-07T20:46:39.175588", "status": "completed"} tags=[]
eye_scans = create_eye_scans(target_objs=[link for link in links])
for eye_scan in eye_scans:
    eye_scan.params[EYE_SCAN_HORZ_STEP].value = 10
    eye_scan.params[EYE_SCAN_VERT_STEP].value = 10
    eye_scan.params[EYE_SCAN_HORZ_RANGE].value = "-0.500 UI to 0.500 UI"
    eye_scan.params[EYE_SCAN_VERT_RANGE].value = "100%"
    eye_scan.params[EYE_SCAN_TARGET_BER].value = 1e-5

    eye_scan.start()
    print(f"Started eye scan {eye_scan}")


# + [markdown] papermill={"duration": 0.006358, "end_time": "2023-06-07T20:46:46.834882", "exception": false, "start_time": "2023-06-07T20:46:46.828524", "status": "completed"} tags=[]
# ## 9 - Wait for all the eye scans to get done

# + papermill={"duration": 0.828215, "end_time": "2023-06-07T20:46:47.669455", "exception": false, "start_time": "2023-06-07T20:46:46.841240", "status": "completed"} tags=[]
eye_scans[0].wait_till_done()
eye_scans[1].wait_till_done()
eye_scans[2].wait_till_done()
eye_scans[3].wait_till_done()

# + [markdown] papermill={"duration": 0.007018, "end_time": "2023-06-07T20:46:47.683833", "exception": false, "start_time": "2023-06-07T20:46:47.676815", "status": "completed"} tags=[]
# ## 10 - View Eye Scan Plot.
#
# This requires Plotly to be installed. See how to install it [here](https://pages.gitenterprise.xilinx.com/chipscope/chipscopy/2020.2/ibert/scan.html#scan-plots)
#
# NOTE - The plot may not display if this notebook is run in Jupyter Lab. For details, see [link](https://plotly.com/python/getting-started/#jupyterlab-support-python-35)


# + papermill={"duration": 1.085739, "end_time": "2023-06-07T20:46:48.776330", "exception": false, "start_time": "2023-06-07T20:46:47.690591", "status": "completed"} tags=[]
eye_scans[0].plot.show()
eye_scans[1].plot.show()
eye_scans[2].plot.show()
eye_scans[3].plot.show()

# + [markdown] papermill={"duration": 0.010248, "end_time": "2023-06-07T20:46:48.805763", "exception": false, "start_time": "2023-06-07T20:46:48.795515", "status": "completed"} tags=[]
# # IBERT Sweep Example
#
# This step begins the sweep example, in which for a single link we iterate through valid property values for our desired TX/RX properties, creating an eye scan for each combination of values and allowing us to identify the property values corresponding to the best eye scan. 
#
# Note that while the eye scan example assumed that no external cable loopback was present, this example was designed to work for devices with external loopback - regardless of the property values we are sweeping over, using internal loopback would likely cause there to not be any appreciable difference in the eye scans generated by the sweep. 
#
# For an example of an external loopback module utilized in this example, click [here](https://www.samtec.com/kits/optics-fpga/hspce-fmcp/).
#
# Finally, for the purposes of this example, we will arbitrarily be using the Quad 205 CH0 TX to Quad 205 CH0 RX link.

# + [markdown] papermill={"duration": 0.010373, "end_time": "2023-06-07T20:46:48.826592", "exception": false, "start_time": "2023-06-07T20:46:48.816219", "status": "completed"} tags=[]
# ## 11 - Cache Properties for Sweep.
#
# As performing a sweep will continuously modify our RX and TX properties, we must begin our sweep by storing the current values in order to restore them after we are done.

# + papermill={"duration": 1.336688, "end_time": "2023-06-07T20:46:50.173677", "exception": false, "start_time": "2023-06-07T20:46:48.836989", "status": "completed"} tags=[]
link = links[0]

#Disabling internal loopback for the reasons outlined above
props = {
    link.rx.property_for_alias[RX_LOOPBACK]: "None"
}
link.rx.property.set(**props)
link.rx.property.commit(list(props.keys()))

print(f"\n----- {link.name} -----")

orig_precursor = list(link.tx.property.refresh(link.tx.property_for_alias[TX_PRE_CURSOR]).values())[0]
orig_postcursor = list(link.tx.property.refresh(link.tx.property_for_alias[TX_POST_CURSOR]).values())[0]
orig_diffswing = list(link.tx.property.refresh(link.tx.property_for_alias[TX_DIFFERENTIAL_SWING]).values())[0]
orig_termvolt = list(link.tx.property.refresh(link.rx.property_for_alias[RX_TERMINATION_VOLTAGE]).values())[0]

print(f"--> Original value of TX Pre Cursor - {orig_precursor}")
print(f"--> Original value of TX Post Cursor - {orig_postcursor}")
print(f"--> Original value of TX Diff Swing - {orig_diffswing}")
print(f"--> Original value of RX Termination Voltage - {orig_termvolt}")

# + [markdown] papermill={"duration": 0.011344, "end_time": "2023-06-07T20:46:50.201046", "exception": false, "start_time": "2023-06-07T20:46:50.189702", "status": "completed"} tags=[]
# ## 12 - Find Properties for Sweep.
# In this step, we find all possible values of the properties we wish to sweep over. For the purposes of this example, only two possible values from each property are used.

# + papermill={"duration": 0.15844, "end_time": "2023-06-07T20:46:50.370231", "exception": false, "start_time": "2023-06-07T20:46:50.211791", "status": "completed"} tags=[]
_, tx_precursor_report = link.tx.property.report(link.tx.property_for_alias[TX_PRE_CURSOR]).popitem()
_, tx_postcursor_report = link.tx.property.report(link.tx.property_for_alias[TX_POST_CURSOR]).popitem()
_, tx_diffswing_report = link.tx.property.report(link.tx.property_for_alias[TX_DIFFERENTIAL_SWING]).popitem()
_, rx_termvolt_report = link.tx.property.report(link.rx.property_for_alias[RX_TERMINATION_VOLTAGE]).popitem()

print(f"\n----- {link.name} -----")
print(f"--> Valid values for TX Pre Cursor - {tx_precursor_report['Valid values']}")
print(f"--> Valid values for TX Post Cursor - {tx_postcursor_report['Valid values']}")
print(f"--> Valid values for TX Diff Swing - {tx_diffswing_report['Valid values']}")
print(f"--> Valid values for RX Termination Voltage - {rx_termvolt_report['Valid values']}")

selected_precurs = tx_precursor_report['Valid values'][0:2]
selected_postcurs = tx_postcursor_report['Valid values'][0:2]
selected_diffswing = tx_diffswing_report['Valid values'][0:2]
selected_termvolt = rx_termvolt_report['Valid values'][0:2]


# + [markdown] papermill={"duration": 0.009756, "end_time": "2023-06-07T20:46:50.390070", "exception": false, "start_time": "2023-06-07T20:46:50.380314", "status": "completed"} tags=[]
# ## 13 - Perform the Sweep.
# Iteration over combinations of property values is performed using the itertools product method, imported in the first cell of this example notebook.
#
# This step displays the resulting eye scans, completing the sweep.

# + papermill={"duration": 58.889641, "end_time": "2023-06-07T20:47:49.289032", "exception": false, "start_time": "2023-06-07T20:46:50.399391", "status": "completed"} tags=[]
combinations = list(product(selected_precurs, 
                selected_postcurs, selected_diffswing, 
                    selected_termvolt))
print(f"-----Total Eye Scans in this Sweep: {len(combinations)}-----")

eye_scans = []
for (precurs, poscurs, difswing, tervolt) in combinations:
    props = {
        link.tx.property_for_alias[TX_PRE_CURSOR]: precurs,
        link.tx.property_for_alias[TX_POST_CURSOR]: poscurs,
        link.tx.property_for_alias[TX_DIFFERENTIAL_SWING]: difswing
    }
    link.tx.property.set(**props)
    link.tx.property.commit(list(props.keys()))
    
    props = {
        link.rx.property_for_alias[RX_TERMINATION_VOLTAGE]: tervolt,
    }
    link.rx.property.set(**props)
    link.rx.property.commit(list(props.keys()))
    
    eye_scan = create_eye_scans(target_objs=link)[0]
    eye_scan.params[EYE_SCAN_HORZ_STEP].value = 10
    eye_scan.params[EYE_SCAN_VERT_STEP].value = 10
    eye_scan.params[EYE_SCAN_HORZ_RANGE].value = "-0.500 UI to 0.500 UI"
    eye_scan.params[EYE_SCAN_VERT_RANGE].value = "100%"
    eye_scan.params[EYE_SCAN_TARGET_BER].value = 1e-5
    eye_scan.start()
    eye_scan.wait_till_done()
    print(f"Finished eye scan {eye_scan}")    
    title_string = f"Pre Cursor: {precurs}, Post Cursor: {poscurs}, Diff Swing: {difswing}," + "<br>" + f" Termination Voltage: {tervolt}" #, Common mode: {commode}
    eye_scans.append([eye_scan,title_string])
    





# + [markdown] papermill={"duration": 0.011881, "end_time": "2023-06-07T20:47:49.314198", "exception": false, "start_time": "2023-06-07T20:47:49.302317", "status": "completed"} tags=[]
# ## 14 - Find the Sweep's Best Eye Scan.
# Using the eye scans that we have created with the sweep, we can deduce which eye scan and its corresponding properties are the most desirable based on many different, often arbitrary metrics. In this example, we will use the proportion of 0 Bit Error Rate within each eye scan. Other ways of identifying eye-scan quality could include calculating the largest width of the dark blue area, or even simply eye-balling the area of the graph. 
#
# In this cell, we calculate our metric using the eye scan's raw data, sorting the resulting eye scans in descending order and printing them out.

# + papermill={"duration": 1.191387, "end_time": "2023-06-07T20:47:50.517377", "exception": false, "start_time": "2023-06-07T20:47:49.325990", "status": "completed"} tags=[]
for scan in eye_scans:
    scan.append(sum([1 for i in scan[0].scan_data.raw.error_count if i == 0])/len(scan[0].scan_data.raw.error_count))

eye_scans.sort(key=lambda x: x[2], reverse=True)

for scan in eye_scans:
    scan[1] += f", Zero Error Rate: {scan[2]}"
    scan[0].plot.show(title=scan[1])

# + [markdown] papermill={"duration": 0.031337, "end_time": "2023-06-07T20:47:50.574369", "exception": false, "start_time": "2023-06-07T20:47:50.543032", "status": "completed"} tags=[]
# ## 15 - Restore Original Property Values.
# After completing the sweep, we must finally restore each property to their original values, which we had cached in step 11.

# + papermill={"duration": 1.957213, "end_time": "2023-06-07T20:47:52.553725", "exception": false, "start_time": "2023-06-07T20:47:50.596512", "status": "completed"} tags=[]
props = {
    link.tx.property_for_alias[TX_PRE_CURSOR]: orig_precursor,
    link.tx.property_for_alias[TX_POST_CURSOR]: orig_postcursor,
    link.tx.property_for_alias[TX_DIFFERENTIAL_SWING]: orig_diffswing,

}    
link.tx.property.set(**props)  
link.tx.property.commit(list(props.keys()))

props = {
    link.rx.property_for_alias[RX_TERMINATION_VOLTAGE]: orig_termvolt,

}    
link.rx.property.set(**props)  
link.rx.property.commit(list(props.keys()))


print(f"Post-sweep value of TX Pre Cursor: {list(link.tx.property.refresh(link.tx.property_for_alias[TX_PRE_CURSOR]).values())[0]}")
print(f"Post-sweep value of TX Post Cursor: {list(link.tx.property.refresh(link.tx.property_for_alias[TX_POST_CURSOR]).values())[0]}")
print(f"Post-sweep value of TX Diff Swing: {list(link.tx.property.refresh(link.tx.property_for_alias[TX_DIFFERENTIAL_SWING]).values())[0]}")
print(f"Post-sweep value of RX Termination Voltage Cursor: {list(link.rx.property.refresh(link.rx.property_for_alias[RX_TERMINATION_VOLTAGE]).values())[0]}")



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py
#     text_representation:
#       extension: .py
#       format_name: light
#       format_version: '1.5'
#       jupytext_version: 1.10.1
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# + [markdown] papermill={"duration": 0.003939, "end_time": "2023-06-08T18:10:58.730296", "exception": false, "start_time": "2023-06-08T18:10:58.726357", "status": "completed"} tags=[]
# <link rel="preconnect" href="https://fonts.gstatic.com">
# <link href="https://fonts.googleapis.com/css2?family=Fira+Code&display=swap" rel="stylesheet">
#
# ### License
#
# <p style="font-family: 'Fira Code', monospace; font-size: 1.2rem">
# Copyright (c) 2021-2022 Xilinx, Inc.<br>
# Copyright (c) 2022-2023 Advanced Micro Devices, Inc.<br><br>
# Licensed under the Apache License, Version 2.0 (the "License");<br>
# you may not use this file except in compliance with the License.<br><br>
# You may obtain a copy of the License at <a href="http://www.apache.org/licenses/LICENSE-2.0"?>http://www.apache.org/licenses/LICENSE-2.0</a><br><br>
# Unless required by applicable law or agreed to in writing, software<br>
# distributed under the License is distributed on an "AS IS" BASIS,<br>
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.<br>
# See the License for the specific language governing permissions and<br>
# limitations under the License.<br>
# </p>
#

# + [markdown] papermill={"duration": 0.002892, "end_time": "2023-06-08T18:10:58.736658", "exception": false, "start_time": "2023-06-08T18:10:58.733766", "status": "completed"} tags=[]
# # IBERT yk scan example

# + [markdown] papermill={"duration": 0.002906, "end_time": "2023-06-08T18:10:58.742482", "exception": false, "start_time": "2023-06-08T18:10:58.739576", "status": "completed"} tags=[]
# ## Description
# This example shows how to interact with the IBERT (Integrated Bit Error Ratio Tester) debug core service via ChipScoPy APIs.
# - Program the ChipScoPy CED design onto the XCVP1202 device on a VPK120 board
# - Verify that the expected IBERT quads are instantiated by the design
# - Run and plot YK scans
#
# ## Requirements
# - Local or remote Xilinx Versal board, such as a VPK120
# - Xilinx hw_server 2022.2 installed and running
# - Xilinx cs_server 2022.2 installed and running
# - Python 3.8 or greater installed
# - ChipScoPy 2022.2 installed
# - Jupyter notebook support installed - Please do so, using the command `pip install chipscopy[jupyter]`
# - Plotting support installed - Please do so, using the command `pip install chipscopy[core-addons]`
# - [External loopback](https://www.samtec.com/kits/optics-fpga/hspce-fmcp/)
# - This example assumes that the device has already been programmed with the example design (ie the debug cores have already been initialized)

# + [markdown] papermill={"duration": 0.002873, "end_time": "2023-06-08T18:10:58.748522", "exception": false, "start_time": "2023-06-08T18:10:58.745649", "status": "completed"} tags=[]
# ## 1 - Initialization: Imports
#
# After this step,
#
# * Required functions and classes are imported
# * Paths to server(s) and files are set correctly

# + papermill={"duration": 0.596266, "end_time": "2023-06-08T18:10:59.347745", "exception": false, "start_time": "2023-06-08T18:10:58.751479", "status": "completed"} tags=[]
import time
from more_itertools import one
import matplotlib.pyplot as plt

from chipscopy import create_session, report_versions, report_hierarchy, get_design_files
from chipscopy.api.ibert import create_yk_scans

# + [markdown] papermill={"duration": 0.003375, "end_time": "2023-06-08T18:10:59.354823", "exception": false, "start_time": "2023-06-08T18:10:59.351448", "status": "completed"} tags=[]
# ## 2 - Create a session and connect to the hw_server and cs_server
#
# The session is a container that keeps track of devices and debug cores.
# After this step,
# - Session is initialized and connected to server(s)
# - Versions are detected and reported to stdout

# + papermill={"duration": 1.37552, "end_time": "2023-06-08T18:11:00.733316", "exception": false, "start_time": "2023-06-08T18:10:59.357796", "status": "completed"} tags=[]
CS_URL = "TCP:localhost:3042"
HW_URL = "TCP:localhost:3121"

# The get_design_files() function tries to find the PDI and LTX files. In non-standard
# configurations, you can put the path for PROGRAMMING_FILE and PROBES_FILE below.
design_files = get_design_files("vpk120/production/chipscopy_ced")

PDI_FILE = design_files.programming_file

print(f"PROGRAMMING_FILE: {PDI_FILE}")

session = create_session(cs_server_url=CS_URL, hw_server_url=HW_URL)
report_versions(session)

# + [markdown] papermill={"duration": 0.003608, "end_time": "2023-06-08T18:11:00.742691", "exception": false, "start_time": "2023-06-08T18:11:00.739083", "status": "completed"} tags=[]
# ## 3 - Program the device with the example design
# After this step,
# * Device is programmed with the example programming file

# + papermill={"duration": 7.655694, "end_time": "2023-06-08T18:11:08.401846", "exception": false, "start_time": "2023-06-08T18:11:00.746152", "status": "completed"} tags=[]
# Typical case - one device on the board - get it.
device = session.devices.filter_by(family="versal").get()
#print(device)
device.program(PDI_FILE)

# + [markdown] papermill={"duration": 0.003756, "end_time": "2023-06-08T18:11:08.410068", "exception": false, "start_time": "2023-06-08T18:11:08.406312", "status": "completed"} tags=[]
# ## 4 - Discover and setup the IBERT core
#
# Debug core discovery initializes the chipscope server debug cores.
#
# After this step,
#
# - The cs_server is initialized and ready for use
# - The first ibert found is used

# + papermill={"duration": 2.561244, "end_time": "2023-06-08T18:11:10.974770", "exception": false, "start_time": "2023-06-08T18:11:08.413526", "status": "completed"} tags=[]
# # Set any params as neded
# params_to_set = {"IBERT.internal_mode": True}
# session.set_param(params_to_set)
device = session.devices.filter_by(family="versal").get()

# Use the first available device and setup its debug cores

print(f"Discovering debug cores...")
device.discover_and_setup_cores(ibert_scan=True)

if len(device.ibert_cores) == 0:
    print("No IBERT core found! Exiting...")
    exit()

# + [markdown] papermill={"duration": 0.004087, "end_time": "2023-06-08T18:11:10.983415", "exception": false, "start_time": "2023-06-08T18:11:10.979328", "status": "completed"} tags=[]
# ## 5 - Print the hierarchy of the IBERT core
# We also ensure that all the quads instantiated by the ChipScoPy CED design are found by the APIs

# + papermill={"duration": 0.191352, "end_time": "2023-06-08T18:11:11.178404", "exception": false, "start_time": "2023-06-08T18:11:10.987052", "status": "completed"} tags=[]
# Use the first available IBERT core from the device
print(f"--> Found {[f'{ibert.name} ({ibert.handle})' for ibert in device.ibert_cores]}\n")

ibert_gtm = one(device.ibert_cores.filter_by(name="IBERT Versal GTM"))

if len(ibert_gtm.gt_groups) == 0:
    print("No GT Groups available for use! Exiting...")
    exit()

print(f"--> Enabled GT Groups - {ibert_gtm.gt_groups}")

gt_group = ibert_gtm.gt_groups.filter_by(name="Quad_204")[0]


# + [markdown] papermill={"duration": 0.003953, "end_time": "2023-06-08T18:11:11.187014", "exception": false, "start_time": "2023-06-08T18:11:11.183061", "status": "completed"} tags=[]
# ## 6 - Define YK Scan Update Method
#
# This method will be called each time the yk scan updates, allowing it to update its graphs in real time. 

# + papermill={"duration": 0.022002, "end_time": "2023-06-08T18:11:11.212836", "exception": false, "start_time": "2023-06-08T18:11:11.190834", "status": "completed"} tags=[]
# %matplotlib notebook

def yk_scan_updates(obj):
    global count, figure, ax, ax2, ax3
    if ax.lines:
        for line in ax.lines:
            line.set_xdata(range(len(obj.scan_data[-1].slicer)))
            line.set_ydata(list(obj.scan_data[-1].slicer))
    else:
        ax.scatter(range(len(obj.scan_data[-1].slicer)), list(obj.scan_data[-1].slicer), color='blue')
    
    if ax2.lines:
        for line2 in ax2.lines:
            ax2.set_xlim(0, ax2.get_xlim()[1] + len(obj.scan_data[-1].slicer))
            line2.set_xdata(list(line2.get_xdata()) + list(range(len(line2.get_xdata()), len(line2.get_xdata()) + len(obj.scan_data[-1].slicer))))
            line2.set_ydata(list(line2.get_ydata()) + list(obj.scan_data[-1].slicer))
    else:
        ax2.scatter(range(len(obj.scan_data[-1].slicer)), list(obj.scan_data[-1].slicer), color='blue')
        
    if ax3.lines:
        for line3 in ax3.lines:
            if len(obj.scan_data) - 1 > ax3.get_xlim()[1]:
                ax3.set_xlim(0, ax3.get_xlim()[1]+10)
            line3.set_xdata(list(line3.get_xdata()) + [len(obj.scan_data) - 1])
            line3.set_ydata(list(line3.get_ydata()) + [obj.scan_data[-1].snr])
    else:
        ax3.plot(len(obj.scan_data) - 1, obj.scan_data[-1].snr)


    figure.canvas.draw_idle()

# + [markdown] papermill={"duration": 0.003723, "end_time": "2023-06-08T18:11:11.220231", "exception": false, "start_time": "2023-06-08T18:11:11.216508", "status": "completed"} tags=[]
# ## 7 - Create YK Scan
#
# This step initializes the YK scan, setting its update method to the method we defined in the last step. 

# + papermill={"duration": 1.500477, "end_time": "2023-06-08T18:11:12.724920", "exception": false, "start_time": "2023-06-08T18:11:11.224443", "status": "completed"} tags=[]
yk = create_yk_scans(target_objs=gt_group.gts[0].rx)[0]

yk.updates_callback = yk_scan_updates

# + [markdown] papermill={"duration": 0.003867, "end_time": "2023-06-08T18:11:12.733329", "exception": false, "start_time": "2023-06-08T18:11:12.729462", "status": "completed"} tags=[]
# ## 8 - Run YK Scan
#
# Initialize the plots and start the YK Scan to begin updating the plots. 
# YK Scan plot should contain three subplots, these plots should look something like:
# ![yk_scan_example.png](./yk_scan_example.png)
# Note: Depending on the hardware setup and external loopback connection, the plot might look different.

# + papermill={"duration": 0.47883, "end_time": "2023-06-08T18:11:13.216139", "exception": false, "start_time": "2023-06-08T18:11:12.737309", "status": "completed"} tags=[]
# %matplotlib notebook

#This sets up the subplots necessary for the 
figure, (ax, ax2, ax3) = plt.subplots(3, constrained_layout = True, num="YK Scan")

ax.set_xlabel("ES Sample")
ax.set_ylabel("Amplitude (%)")
ax.set_xlim(0,2000)
ax.set_ylim(0,100)
ax.set_yticks(range(0, 100, 20))
ax.set_title("Slicer eye")

ax2.set_xlabel("Count")
ax2.set_ylabel("Amplitude (%)")
ax2.set_xlim(0,2000)
ax2.set_ylim(0,100)
ax2.set_yticks(range(0, 100, 20))
ax2.set_title("Histogram")

ax3.set_xlabel("SNR Sample")
ax3.set_ylabel("SNR (dB)")
ax3.set_xlim(0,10)
ax3.set_ylim(-10,100)
ax3.set_title("Signal-to-Noise Ratio")

yk.start()

# + [markdown] papermill={"duration": 0.005069, "end_time": "2023-06-08T18:11:13.226526", "exception": false, "start_time": "2023-06-08T18:11:13.221457", "status": "completed"} tags=[]
# ## 9 - Stop YK Scan
# Stops the YK scan from running.

# + papermill={"duration": 2.034811, "end_time": "2023-06-08T18:11:15.267022", "exception": false, "start_time": "2023-06-08T18:11:13.232211", "status": "completed"} tags=[]
yk.stop()
